"""
This provides functions for parsing information coming from the ANT+ USB dongle sensors.
Particularly using the Garmin Cadence/Forward speed sensor.

It is not being used in the SOFIE project because the sample speed is not fast enough.
"""
import logging
import math
from ant.core import message as m
from ant.core.exceptions import MessageError
import sofiehdfformat.core.SofieHdfFormatExceptions as exceptions
from sofiehdfformat.core.SofieHdfFormatExceptions import ParserCanOnlyParseADict
from sofiehdfformat.core.SofieHdfFormatExceptions import ParserIncorrectDictFormat
__author__="cooke"
__date__ ="$29-Sep-2011 14:10:44$"


Ts_speedcad = 8086.0/32768.0
'''
Period in seconds
'''
ROLL=65536.0
'''
The sample rate of Cadence/Speed sensor.
'''
def rollover(returnData,previousRow,valueToRoll):
    if returnData[valueToRoll] < \
                previousRow[valueToRoll]:
            amount = int(math.ceil(previousRow[valueToRoll]/ROLL)*ROLL)
            returnData[valueToRoll] += amount
"""
Implements a Parser that parses a GeFile sample into an array.

"""
def parse_sample(sampleDict):
    """
    Parses one ANT packet string into a list of the raw bytes and returns.

    This parsed only ChannelBroadcastDataMessage's and returns only the payload 
    as a list. If the length of the sampleString equals ANTDATALENGTH+ANTPACKETLEN
    then the remaining part of the string is assumed to be the device timestamp 
    int converted to STRING in UNIXTIME.

    Returns a list with the packet bytes in a list and the timesamp the last element.
    """
    if not isinstance(sampleDict,dict):
        raise ParserCanOnlyParseADict()
    if not 'packet' in sampleDict or 'timestamp' not in sampleDict:
        raise ParserIncorrectDictFormat()

    hf = m.Message()
    try:
        msg = hf.getHandler(sampleDict['packet'])
        if isinstance(msg, m.ChannelBroadcastDataMessage):
            logging.debug('Channel BroadcastDataMessage: Payload Information:'+
                 str(type(msg.payload))+' '+str(msg.payload));
            #if len(msg)>ANTDATALENGTH+ANTPACKETLEN
            msg.payload = [ord(d) for d in msg.payload]
            return {
                'ChanID':msg.payload[0],
                'BikeCadenceEventTimeLSB':msg.payload[1],
                'BikeCadenceEventTimeMSB':msg.payload[2],
                'CumulativeCadenceRevolutionCountLSB':msg.payload[3],
                'CumulativeCadenceRevolutionCountMSB':msg.payload[4],
                'BikeSpeedEventTimeLSB':msg.payload[5],
                'BikeSpeedEventTimeMSB':msg.payload[6],
                'CumulativeSpeedRevolutionCountLSB':msg.payload[7],
                'CumulativeSpeedRevolutionCountMSB':msg.payload[8],
                'Timestamp':sampleDict['timestamp'],
                }
            logging.debug('Not a broadcast.')
        return [];
    except MessageError, e:
        if e.internal == "CHECKSUM":
            logging.debug('Checksum Error:');
            raise exceptions.ChecksumParseError
            #buffer_ = buffer_[ord(buffer_[1]) + 4:]
        else:
            logging.debug('Other Error:'+e.internal);
            raise exceptions.UnknownParseError
    #logging.debug('The Parsed Msg: '+str(msg)

"""
Implements a Parser that parses a AntRawData sample from a speed cadence meter.

It returns a dict with the format
dict(
    'BikeCadenceEventTime':,
    'CumulativeCadenceRevolutionCount':,
    'CumulativeSpeedRevolutionCount':
    'Timestamp':
    ).

"""
def parse_sample_interpret(sampleData,previousRow=None):
    if isinstance(sampleData, list):
        sampleData = {
        'ChanID':sampleData[0],
        'BikeCadenceEventTimeLSB':sampleData[1],
        'BikeCadenceEventTimeMSB':sampleData[2],
        'CumulativeCadenceRevolutionCountLSB':sampleData[3],
        'CumulativeCadenceRevolutionCountMSB':sampleData[4],
        'BikeSpeedEventTimeLSB':sampleData[5],
        'BikeSpeedEventTimeMSB':sampleData[6],
        'CumulativeSpeedRevolutionCountLSB':sampleData[7],
        'CumulativeSpeedRevolutionCountMSB':sampleData[8],
        'Timestamp':sampleData[9],
        }

    returnData = {'BikeCadenceEventTime':
        int(sampleData['BikeCadenceEventTimeMSB'])*256+\
        int(sampleData['BikeCadenceEventTimeLSB']),
        'CumulativeCadenceRevolutionCount':
        int(sampleData['CumulativeCadenceRevolutionCountMSB'])*256+\
        int(sampleData['CumulativeCadenceRevolutionCountLSB']),
        'BikeSpeedEventTime':
        int(sampleData['BikeSpeedEventTimeMSB'])*256+\
        int(sampleData['BikeSpeedEventTimeLSB']),
        'CumulativeSpeedRevolutionCount':
        int(sampleData['CumulativeSpeedRevolutionCountMSB'])*256+\
        int(sampleData['CumulativeSpeedRevolutionCountLSB']),
        'Timestamp':sampleData['Timestamp'],
        'BikeCadenceHoldTimestamp':sampleData['Timestamp'],
        'BikeSpeedHoldTimestamp':sampleData['Timestamp'],
        }

    if previousRow:
        #Roll over
        rollover(returnData,previousRow,'BikeSpeedEventTime');
        rollover(returnData,previousRow,'BikeCadenceEventTime');
        rollover(returnData,previousRow,'CumulativeCadenceRevolutionCount');
        rollover(returnData,previousRow,'CumulativeSpeedRevolutionCount');
        
        cadenceDiff = returnData['BikeCadenceEventTime']-\
            previousRow['BikeCadenceEventTime']
        cadenceTimeStampDiff = returnData['BikeCadenceHoldTimestamp']-\
            previousRow['BikeCadenceHoldTimestamp']

        logging.debug('TIME STAMP DIFF: '+str(cadenceTimeStampDiff)+' '+\
            str(5*Ts_speedcad))
        
        if cadenceDiff>0:
            logging.debug('New Cadence occurred: Revs:'+
                str(cadenceDiff))
            #1024*60*revcount/(time for distance in 1/1024 seconds)=revs/minute
            returnData['BikeCadence']= 1024*60*\
                float(returnData['CumulativeCadenceRevolutionCount'] -
                previousRow['CumulativeCadenceRevolutionCount'])\
                /float(returnData['BikeCadenceEventTime']-\
                previousRow['BikeCadenceEventTime'])
            logging.debug(returnData['BikeCadence'])
        elif (cadenceDiff==0) and (cadenceTimeStampDiff<5*Ts_speedcad):
            logging.debug('HOLDING CADENCE')
            #Holding for max of 5 samples:
            returnData['BikeCadence'] = previousRow['BikeCadence']
            returnData['BikeCadenceHoldTimestamp'] = \
                previousRow['BikeCadenceHoldTimestamp']
        else:
            returnData['BikeCadence'] = 0.0
            returnData['BikeCadenceHoldTimestamp'] = returnData['Timestamp']

        speedDiff = returnData['BikeSpeedEventTime']-\
            previousRow['BikeSpeedEventTime']
        speedTimeStampDiff = returnData['BikeSpeedHoldTimestamp']-\
            previousRow['BikeSpeedHoldTimestamp']

        if speedDiff>0.0:
            #1024*distance/(time for distance in 1/1024 seconds)=m/s
            returnData['BikeSpeed']= 1024*\
                float(returnData['CumulativeSpeedRevolutionCount'] -
                previousRow['CumulativeSpeedRevolutionCount'])\
                /float(returnData['BikeSpeedEventTime']-\
                previousRow['BikeSpeedEventTime'])
        elif (speedDiff==0) and \
            (speedTimeStampDiff<5*Ts_speedcad):
            logging.debug('HOLDING SPEED')
            #Holding for max of 5 samples:
            returnData['BikeSpeed'] = previousRow['BikeSpeed']
            returnData['BikeSpeedHoldTimestamp'] = previousRow['BikeSpeedHoldTimestamp']
        else:
            returnData['BikeSpeed'] = 0.0
            returnData['BikeSpeedHoldTimestamp'] = returnData['Timestamp']
    else:
        returnData['BikeSpeed'] = 0.0
        returnData['BikeCadence'] = 0.0
    #logging.debug(returnData)
    return returnData


