import logging
"""
This script provides a Duck typed file reader for c3d files. It was
modified from c3d_converter.py Taken from:
http://blog.noisygecko.com/c3d_convert.py


Convert a "c3d" format motion capture file into a
comma separated file. 
This code was taken from "c3d_import.py" by
Jean-Baptiste PERIN (jb_perin(at)yahoo.fr)
"""
import __builtin__
import math
import os
import string
import struct
import sys
class Marker:
    def __init__(self, x, y, z):
        self.x=0.0
        self.y=0.0
        self.z=0.0
        self.Timestamp = 0.0
        self.__iterNumber = 0
    def __iter__(self):
        self.__iterNumber = 0
        return self;
    def next(self):
        self.__iterNumber += 1;
        if self.__iterNumber == 1:
            return self.x
        elif self.__iterNumber == 2:
            return self.y
        elif self.__iterNumber == 3:
            return self.z
        elif self.__iterNumber == 4:
            return self.Timestamp
        else:
            #logging.debug('Marker stoping iteration:'+str(self.__iterNumber))
            raise StopIteration

    def __repr__(self):
        return str("x,"+str(self.x) +",y," + str(self.y)+",x,"+\
        str(self.z)+",Timestamp,"+str(self.Timestamp)+"]")


class ParameterGroup:
  def __init__(self, nom, description, parameter):
    self.name = nom
    self.description = description
    self.parameter = parameter

  def __repr__(self):
    return self.name, " ", self.description, " ", self.parameter



class Parameter:
  def __init__(self, name, datatype, dim, data, description):
    self.name = name
    self.datatype = datatype
    self.dim = dim
    self.data = data
    self.description = description
 
    def __repr__(self):
      return self.name, " ", self.description, " ", self.dim


def getNumber(str, length):
    sum = 0
#    logging.debug(length)
#    logging.debug(type(str))
    for i in range(length):
        #sum = (sum << 8) + ord(str[i]) for big indian    
        sum = sum + ord(str[i])*(2**(8*i))
    return sum, str[length:]


def getFloat(theString,proctype=-1):
    if proctype == 2:
        #logging.debug('Converting')
        return uint32le_to_VAXF(theString[0:4]),theString[4:]
    return struct.unpack('f',theString[0:4])[0], theString[4:]

def uint32le_to_VAXF(theString):
    uint32le = struct.unpack('I',theString[0:4])[0]
    return uint32le_to_VAXF_orig(uint32le)

def uint32le_to_VAXF_orig( uint32le ):
    """
uint32le_to_VAXF_orig Converts from IEEELE (UINT32) to VAXF (single precision)

Uses http://www.opengroup.org/onlinepubs/9629399/chap14.htm#tagfcjh_20
    """
    A = 2   ;#VAX specific
    B = 128 ;#VAX specific
    C = 0.5 ;#VAX specific
# Flip the upper and lower bits (based on how Vax data storage format)
# VAX      <-----WORD1-----><-----WORD2----->
# IEEE-LE  <-----WORD2-----><-----WORD1----->
    #logging.debug('Uint23le:'+str(uint32le))
    #logging.debug("Total: "+str(hex(uint32le)))

    word2  = uint32le&0xFFFF0000;#mask FFFF0000
    #logging.debug("Word2: "+bin(word2)+' : '+str(word2))
    word1  = uint32le&0x0000FFFF;#mask 0000FFFF
    #logging.debug("Word1: %s "%bin(word1))
    vaxInt = (word1<<16)|(word2>>16);
    #logging.debug("VaxInt:"+hex(vaxInt)+' : '+str(vaxInt))
# Pull out the sign, exponent, and fractional component
# VAX FLOAT BYTES  <-----WORD1----><-----WORD2---->
# VAX FLOAT BITS   0123456789ABCDEF0123456789ABCDEF
# Sign Exp Fract   SEEEEEEEEFFFFFFFFFFFFFFFFFFFFFFF

    S = int((vaxInt>>31)&0b1);#2147483648=hex2dec('80000000')
    E = int((vaxInt>>23)&0b11111111);#2139095040=hex2dec('7F800000')
    F = int(vaxInt&0x7FFFFF);#   8388607=hex2dec('007FFFFF')
    #logging.debug("S: %s"% bin(S) )
    #logging.debug("S: %i"% S )
    #logging.debug("E: %s "%bin(E))
    #logging.debug("F: %s "%bin(F))
    M = C+float(F)/16777216;#VAX Specific 16777216=2^24
    returnVal = math.pow(-1,S) * M * math.pow(A,(float(E)-B));#Generic
    #logging.debug("FINAL FLOAT: %f "%returnVal)
    return returnVal

def open(filename,mode='r'):
    return C3DFile(filename,mode);

class C3DFile(object):

    def __init__(self,filename,mode):

    # Input:        filename - file (including path) to be read
    #
    # Variable:
    # Markers            3D-marker data [Nmarkers x self.NvideoFrames x Ndim(=3)]
    # self.VideoFrameRate     Frames/sec
    # AnalogSignals      Analog signals [Nsignals x NanalogSamples ]
    # AnalogFrameRate    Samples/sec
    # Event              Event(Nevents).time ..value  ..name
    # ParameterGroup     ParameterGroup(Ngroups).Parameters(Nparameters).data ..etc.
    # CameraInfo         MarkerRelated CameraInfo [Nmarkers x self.NvideoFrames]
    # ResidualError      MarkerRelated ErrorInfo  [Nmarkers x self.NvideoFrames]

      
      self.VideoFrameRate=0;
      self.ParameterGroups=[];
      
      logging.info("*********************")
      logging.info( "**** Opening File ***")
      logging.info( "*********************")

      logging.info( "FileName = "+filename)
      self.fileId=__builtin__.open(filename,'rb'); # native format (PC-intel)
      content = self.fileId.read();
      content_memory = content

      NrecordFirstParameterblock, content = getNumber(content,1)
      # Reading record number of parameter section
      key, content = getNumber(content,1)
      if key!=80:
        logging.info( 'File: '+filename+' does not comply to the C3D format')
        self.fileId.close()
        return
      content = content[512*(NrecordFirstParameterblock-1)+1:]
      self.proctype,content =getNumber(content,1)
      self.proctype = self.proctype-83
      # proctype: 1(INTEL-PC); 2(DEC-VAX); 3(MIPS-SUN/SGI)
      logging.info( "*************************")
      logging.info( "**** Processor coding ***")
      logging.info( "*************************")

      if self.proctype==1:
        logging.info( "Intel-PC")
      elif self.proctype==2:
        logging.info( "DEC-VAX")
      elif self.proctype==3:
        logging.info( "MIPS-SUN/SGI")
      else:
        logging.info( "unknown processor type")


      #if self.proctype==2,
      #    fclose(self.fileId);
      #    self.fileId=fopen(filename,'r','d'); % DEC VAX D floating point and VAX ordering
      #end
      logging.info( "***********************")
      logging.info( "**** Reading Header ***")
      logging.info( "***********************")

      # ###############################################
      # ##                                           ##
      # ##    read header                            ##
      # ##                                           ##
      # ###############################################

      #%NrecordFirstParameterblock=fread(self.fileId,1,'int8');     % Reading record number of parameter section
      #%key1=fread(self.fileId,1,'int8');                           % key = 80;

      content = content_memory

      #
      #fseek(self.fileId,2,'bof');
      content = content[2:]

      #
      self.Nmarkers, content=getNumber(content, 2)
      NanalogSamplesPerVideoFrame, content = getNumber(content, 2)

      StartFrame,  content = getNumber(content, 2)
      EndFrame,  content = getNumber(content, 2)

      MaxInterpolationGap,  content = getNumber(content, 2)

      self.Scale, content = getFloat(content,self.proctype)

      NrecordDataBlock,  content = getNumber(content, 2)

      self.NanalogFramesPerVideoFrame,  content = getNumber(content, 2)


      if self.NanalogFramesPerVideoFrame > 0:
        self.NanalogChannels=NanalogSamplesPerVideoFrame/\
            self.NanalogFramesPerVideoFrame
      else:
        self.NanalogChannels=0

      self.VideoFrameRate, content = getFloat(content,self.proctype)

      AnalogFrameRate=self.VideoFrameRate*self.NanalogFramesPerVideoFrame

      logging.info( "self.NanalogFramesPerVideoFrame= "+\
        str(self.NanalogFramesPerVideoFrame))
      logging.info( "AnalogFrameRate= "+str(AnalogFrameRate))
      logging.info( "self.VideoFrameRate= "+str(self.VideoFrameRate))
      logging.info( "self.Scale= "+str(self.Scale))
      logging.info( "self.Nmarkers= "+str(self.Nmarkers))
      logging.info( "StartFrame= "+str(StartFrame))
      logging.info( "EndFrame= "+str(EndFrame))

      logging.info( "***********************")
      logging.info( "**** Reading Events ...")
      logging.info( "***********************")

      content = content_memory
      content = content[298:] #bizarre .. ce devrait etre 150 selon la doc

      EventIndicator,  content = getNumber(content, 2)


      EventTime=[]
      EventValue=[]
      EventName=[]

      logging.info( "EventIndicator = "+str(EventIndicator))
      if EventIndicator==12345:
        Nevents,  content = getNumber(content, 2)
        logging.info("Nevents= "+str(Nevents))
        content = content[2:]
        if Nevents>0:
          for i in range(Nevents):
            letime, content = getFloat(content,self.proctype)
            EventTime.append(letime)
          content = content_memory
          content = content[188*2:]
          for i in range(Nevents):
            lavalue, content = getNumber(content, 1)
            EventValue.append(lavalue)
          content = content_memory
          content = content[198*2:]
          for i in range(Nevents):
            lenom = content[0:4]
            content = content[4:]
            EventName.append(lenom)


      logging.info( "***************************")
      logging.info( "**** Reading Parameters ...")
      logging.info( "***************************")

      content = content_memory
      content = content[512*(NrecordFirstParameterblock-1):]

      self.ParameterGroups = []
      ParameterNumberIndex = []
      #
      dat1, content = getNumber(content, 1)
      key2, content = getNumber(content, 1)

      NparameterRecords, content = getNumber(content, 1)
      logging.info( "NparameterRecords="+str(NparameterRecords))
      proctype1,content =getNumber(content,1)
      proctype1 = proctype1-83
      # proctype: 1(INTEL-PC); 2(DEC-VAX); 3(MIPS-SUN/SGI)

      for i in range(NparameterRecords):
        leparam = ParameterGroup(None, None, [])
        self.ParameterGroups.append(leparam)
        ParameterNumberIndex.append(0)


      #
      #
      Ncharacters, content = getNumber(content, 1)
      logging.debug('The NUMBER OF CHARACTERS:'+str(Ncharacters))
      if Ncharacters>=128:
        Ncharacters = -(2**8)+(Ncharacters)
      GroupNumber, content = getNumber(content, 1)
      if GroupNumber>=128:
        GroupNumber = -(2**8)+(GroupNumber)
      #logging.info( "GroupNumber = ", GroupNumber

      while Ncharacters > 0:
        if GroupNumber<0 and abs(GroupNumber) < len(self.ParameterGroups):
          GroupNumber=abs(GroupNumber)
          GroupName = content[0:Ncharacters]
          content = content[Ncharacters:]
          logging.debug("Group Name ="+GroupName+" Group Number = "+\
            str(GroupNumber)+"Len self.ParameterGroups:"+\
                str(len(self.ParameterGroups)))
          self.ParameterGroups[GroupNumber].name = GroupName
          #logging.info( "ParameterGroupName =", GroupName
          offset, content = getNumber(content, 2)
          deschars, content = getNumber(content, 1)
          GroupDescription = content[0:deschars]
          content = content[deschars:]
          self.ParameterGroups[GroupNumber].description = GroupDescription
      #
          ParameterNumberIndex[GroupNumber]=0
          content = content[offset-3-deschars:]
        elif abs(GroupNumber) >= len(self.ParameterGroups):
            logging.debug('NOT DECODING GROUP.')
        else:

          ParameterNumberIndex[GroupNumber]=ParameterNumberIndex[GroupNumber]+1
          ParameterNumber=ParameterNumberIndex[GroupNumber]
          #logging.info( "ParameterNumber=", ParameterNumber
          self.ParameterGroups[GroupNumber].parameter.append(
            Parameter(None, None, [], [], None))
          ParameterName = content[0:Ncharacters]
          content = content[Ncharacters:]
          #logging.info( "ParameterName = ",ParameterName
          if len(ParameterName)>0:
            self.ParameterGroups[GroupNumber].\
                parameter[ParameterNumber-1].name=ParameterName
          offset, content = getNumber(content, 2)
          filepos = len(content_memory)-len(content)
          nextrec = filepos+offset-2

          theType, content=getNumber(content, 1)
          if theType>=128:
            theType = -(2**8)+theType
          self.ParameterGroups[GroupNumber].\
            parameter[ParameterNumber-1].type=theType

          dimnum, content=getNumber(content, 1)
          if dimnum == 0:
            datalength = abs(theType)
          else:
            mult=1
            dimension=[]
            for j in range (dimnum):
              ladim, content = getNumber(content, 1)
              dimension.append(ladim)
              mult=mult*dimension[j]
              self.ParameterGroups[GroupNumber].\
                parameter[ParameterNumber-1].dim.append(dimension[j])
            datalength = abs(theType)*mult

          #logging.info( "ParameterNumber = ", ParameterNumber, " Group Number = ", GroupNumber

          if theType==-1:
            data = ""
            wordlength=dimension[0]
            if dimnum==2 and datalength>0:
              for j in range(dimension[1]):
                data=string.rstrip(content[0:wordlength])
                content = content[wordlength:]
                self.ParameterGroups[GroupNumber].\
                    parameter[ParameterNumber-1].data.append(data)
            elif dimnum==1 and datalength>0:
                data=content[0:wordlength]
                self.ParameterGroups[GroupNumber].\
                    parameter[ParameterNumber-1].data.append(data) # ???
            if string.rstrip(ParameterName) == "LABELS" and \
                    string.rstrip(GroupName) == "POINT":
              self.markerNames = self.ParameterGroups[GroupNumber].\
                parameter[ParameterNumber-1].data
              logging.info( "POINT = "+str(self.markerNames)+' '+str(type(self.markerNames)))
            elif string.rstrip(ParameterName) == "LABEL_PREFIXES" and \
                string.rstrip(GroupName) == "SUBJECTS":
              logging.info( "SUBJECTS = "+\
                str( self.ParameterGroups[GroupNumber].\
                    parameter[ParameterNumber-1].data))
            else:
              #logging.info(  self.ParameterGroups[GroupNumber].parameter[ParameterNumber-1].data
              pass


          elif theType == 1:
            data = []
            Nparameters=datalength/abs(theType)
            logging.debug("Nparameters="+str(Nparameters))
            for i in range(Nparameters):
              ladata,content = getNumber(content, 1)
              data.append(ladata)
            self.ParameterGroups[GroupNumber].\
                parameter[ParameterNumber-1].data=data
            #logging.info( self.ParameterGroups[GroupNumber].parameter[ParameterNumber-1].data

            #logging.info( "type boolean"
          elif theType == 2 and datalength>0:
            data = []
            Nparameters=datalength/abs(theType)
            for i in range(Nparameters):
              ladata,content = getNumber(content, 2)
              data.append(ladata)
            #self.ParameterGroups[GroupNumber].parameter[ParameterNumber-1].data=data
            if dimnum>1:
              #???? logging.info( "arg je comprends pas"
              self.ParameterGroups[GroupNumber].\
                parameter[ParameterNumber-1].data=data
              #???self.ParameterGroups[GroupNumber].parameter[ParameterNumber-1].data=reshape(data,dimension)
            else:
              self.ParameterGroups[GroupNumber].\
                parameter[ParameterNumber-1].data=data
            #logging.info( self.ParameterGroups[GroupNumber].parameter[ParameterNumber-1].data

            #pass
            #logging.info( "type integer"
          elif theType == 4 and datalength>0:
            data = []
            Nparameters=datalength/abs(theType)
            for i in range(Nparameters):
              ladata,content = getFloat(content,self.proctype)
              data.append(ladata)
            if dimnum>1:
              self.ParameterGroups[GroupNumber].\
                parameter[ParameterNumber-1].data=data
              #logging.info( "arg je comprends pas"
              #???self.ParameterGroups[GroupNumber].parameter[ParameterNumber-1].data=reshape(data,dimension)
            else:
              self.ParameterGroups[GroupNumber].\
                parameter[ParameterNumber-1].data=data
            #logging.info( self.ParameterGroups[GroupNumber].parameter[ParameterNumber-1].data
          else:
            #logging.info( "error"
            pass
          deschars, content= getNumber(content, 1)
          if deschars>0:
            description = content[0:deschars]
            content = content[deschars:]
            self.ParameterGroups[GroupNumber].\
                parameter[ParameterNumber-1].description=description

          content = content_memory
          content = content[nextrec:]
        logging.debug('LEN CONTENT:'+str(len(content)))
        if not content:
            break
        Ncharacters,content = getNumber(content, 1)
        logging.debug('The NUMBER OF CHARACTERS:'+str(Ncharacters))
        if Ncharacters>=128:
          Ncharacters = -(2**8)+(Ncharacters)
        GroupNumber,content = getNumber(content, 1)
        if GroupNumber>=128:
          GroupNumber = -(2**8)+(GroupNumber)
        #logging.info( "GroupNumber = ", GroupNumber



      ## ###############################################
      ## ##                                           ##
      ## ##    read data block                        ##
      ## ##                                           ##
      ## ###############################################
      ##  Get the coordinate and analog data
      #

      content = content_memory
      self.content = content[(NrecordDataBlock-1)*512:]
      self.ptr_read = 0
      self.NvideoFrames = EndFrame - StartFrame + 1
      self.currentMarker = 0
      logging.info( "NVideoFrames = "+str(self.NvideoFrames))

    def close(self):
        self.fileId.close()

    def getMarkerNames(self):
      return self.markerNames

    def getVideoSamplingRate(self):
       return self.VideoFrameRate

    def getVideoFrames(self):
       return self.NvideoFrames

    def getScale(self):
       return self.Scale

    def readline(self,size=0):
    # Markers            3D-marker data [Nmarkers x self.NvideoFrames x Ndim(=3)]
    # CameraInfo         MarkerRelated CameraInfo [Nmarkers x self.NvideoFrames]
    # ResidualError      MarkerRelated ErrorInfo  [Nmarkers x self.NvideoFrames]

        Markers= {}
        ResidualError = {}
        CameraInfo= {}
        for markerName in self.markerNames:
          Markers[markerName] = Marker(0.0,0.0,0.0)
          ResidualError[markerName] = 0
          CameraInfo[markerName] = 0
        #logging.debug( "***************************")
        #logging.debug( "**** Reading DataBlock ....")
        #logging.debug( "***************************")
        if self.ptr_read >= len(self.content):
            logging.debug( "Nothin More to read.")
            return False

        self.currentMarker += 1
        self.currentTimeStamp = self.currentMarker/self.VideoFrameRate


        if self.Scale < 0.0:
            #logging.info( "*",
            for markerName in self.markerNames:
                x = getFloat(self.content[self.ptr_read:self.ptr_read+4],
                    self.proctype)[0]
                self.ptr_read+=4
                y = getFloat(self.content[self.ptr_read:self.ptr_read+4],
                    self.proctype)[0]
                self.ptr_read+=4
                z = getFloat(self.content[self.ptr_read:self.ptr_read+4],
                    self.proctype)[0]
                self.ptr_read+=4
                Markers[markerName].x = x#*self.Scale
                Markers[markerName].y = y#*self.Scale
                Markers[markerName].z = z#*self.Scale
                Markers[markerName].Timestamp = self.currentTimeStamp#*self.Scale
                a= getFloat(self.content[self.ptr_read:self.ptr_read+4],
                    self.proctype)[0]
                self.ptr_read+=4
                a = int(a)
                highbyte = int(a/256)
                lowbyte=a-highbyte*256
                CameraInfo[markerName] = highbyte
                ResidualError[markerName] = lowbyte*abs(self.Scale)
                #if i< 2:
                  #logging.info( Markerss[i][j]
            self.ptr_read+=self.NanalogFramesPerVideoFrame*self.NanalogChannels*2
        else:
            #UNTESTED
            for makerName in self.markerNames:
                #x, self.content = getNumber(self.content,2)
                x = ord(self.content[self.ptr_read]) + \
                    ord(self.content[self.ptr_read+1])*(2**8)
                self.ptr_read+=2
                if x > 32768:
                  x=-(2**16)+(x)
                #y, self.content = getNumber(self.content,2)
                y = ord(self.content[self.ptr_read]) + \
                    ord(self.content[self.ptr_read+1])*(2**8)
                self.ptr_read+=2
                if y > 32768:
                  y=-(2**16)+(y)
                #z, self.content = getNumber(self.content,2)
                z = ord(self.content[self.ptr_read]) + \
                    ord(self.content[self.ptr_read+1])*(2**8)
                self.ptr_read+=2
                if z > 32768:
                  z=-(2**16)+(z)
                Markers[makerName].x = x*self.Scale
                Markers[makerName].y = y*self.Scale
                Markers[makerName].z = z*self.Scale
                Markers[markerName].Timestamp = self.currentTimeStamp#*self.Scale
                #if i< 2:
                #  logging.info( Markers[i][j]
                ResidualError[makerName], self.content = getNumber(self.content, 1)
                CameraInfo[makerName], self.content = getNumber(self.content, 1)
            self.ptr_read += self.NanalogFramesPerVideoFrame*self.NanalogChannels*2
        #return [Markers,ResidualError,CameraInfo]
        return Markers
          #for j in range (self.NanalogFramesPerVideoFrame):
          #  for k in range(self.NanalogChannels):
          #    val, self.content = getNumber(self.content, 2)
              #AnalogSignals(j+self.NanalogFramesPerVideoFrame*(i-1),1:self.NanalogChannels)=val
    def __iter__(self):
        return self

    def next(self):
        data = self.readline()
        if data == False or data == None:
            raise StopIteration
        return data
