function test_suite = test_arrayToString
%test_arrayToString Unit test for arrayToString.

%   Steven L. Eddins
%   Copyright 2009 The MathWorks, Inc.

initTestSuite;

function test_readLeafData
filename = 'simple-test-data.h5';
reader = h5Reader(filename,'/test/goldenembeded/');
data = reader.readLeafData('AntRawData');
assertEqual(size(data),[52    10]);
data = reader.readLeafData('AntRawData',true);
assertEqual(size(data),[52    10]);
data = reader.readLeafData('AntRawData',false);
assertEqual(class(data),'struct');
data = reader.leafData2Array(data);
assertEqual(size(data),[52 10]);
data = reader.readLeafData('AntSpedData',false);
assertEqual(data,false);


function test_readAdamsData
filename = 'simple-test-data.h5';
reader = adamsReader(filename,'/adams/');
data = reader.readData();
assertEqual(data(501,:),[0.0338851400000000,-0.0759644700000000,20]);
data = reader.readData(false);
assertEqual(data.RollAngle(501),0.0338851400000000);
assertEqual(data.SteeringAngle(501),-0.0759644700000000);
assertEqual(data.Time(501),20);

function test_readAntData
format long;
filename = 'simple-test-data.h5';
reader = antReader(filename,'/sparktest/');
data = reader.readRawData();
dataSize = size(data)
data(42,1)
class(data(42,1))
class(313.0)
assertEqual(data(42,1),313);
assertEqual(data(42,5),1.336465792000000e+09);
data = reader.readData();
dataSize = size(data)
data(42,1)
class(data(42,1))
class(313.0)
assertEqual(data(42,5),1.336465792000000e+09);
assertEqual(data(42,1),313);
assertEqual(data(42,2),16438);

function test_readViconData
filename = 'simple-test-data.h5';
reader = c3dReader(filename,'/vicon/');
data1 = reader.readMarker('RBO');
data2 = reader.readMarker('LBO');
data3 = reader.readMarker('FON');
assertEqual(size(data1),size(data2));
assertEqual(size(data2),size(data3));

function test_readPromoveData
format long
filename = 'simple-test-data.h5';
reader = promoveReader(filename,'/promovetest/');
data = reader.readNodeData(1);
data2 = reader.readNodeData(2);
assertEqual(data2(1,:),[0.784450881000000,0.148633709000000,0.118747481000000,0.590291345000000,331901]);
assertEqual(data2(14484,:),[0.681772478000000,0.142012363000000,0.129500907000000,0.705867049000000,361216]);
assertEqual(data(1,:),[0.696001388000000,-0.00362268554000000,-0.0244893507000000,0.717613556000000,331907;]);
assertEqual(data(1,:),[0.696001388000000,-0.00362268554000000,-0.0244893507000000,0.717613556000000,331907]);
%There are some rounding errors.
assertEqual(data(14470,:),[0.696884605000000,-0.0084978914500000,-0.0231975389000000,0.716757635000000,361215]);
assertFalse(size(data,1)==size(data2,1))

filename = 'simple-test-data.h5';
reader = promoveReader(filename,'/newpromove/');
[data, sync] = reader.readNodeData(1);
data2 = reader.readNodeData(2);
assertFalse(any(sync))

reader = promoveReader(filename,'/newpromovewithsync/');
[data, sync] = reader.readNodeData(1);
[data2,sync2] = reader.readNodeData(2);
assertTrue(any(sync))
assertTrue(any(sync2))
assertEqual(size(sync,2),size(data,1))
assertEqual(size(sync2,2),size(data2,1))

[data3, sync] = reader.readNodeData(-1);
[data4] = reader.readRawNodeData();
assertEqual(size(data3,1),size(data,1)+size(data2,1));
assertEqual(data3,data4);

function testWriteLeafData
filename = 'simple-test-data-write.h5';
reader = h5Writer(filename,'test');
% reader.writeLeafData('AntRawData',[1 2;3 4]);
    
writeData = [1 2 3; 4 5 6]
reader.writeLeafData('AntRawData',writeData);
data = reader.readLeafData('AntRawData',true);
assertEqual(data,writeData);

writeData = [1 2 3; 4 5 6; 7 8 9];
reader.writeLeafData('AntRawData',writeData);
data = reader.readLeafData('AntRawData',true)
assertEqual(data,writeData);


writeData = [1 2; 3 4; 5 6];
reader.writeLeafData('AntRawData',writeData);
data = reader.readLeafData('AntRawData',true)
assertEqual(data,writeData);

writeData = [1 2; 3 4; 5 7];
reader.setRunName('again')
reader.writeLeafData('Again',writeData);
data = reader.readLeafData('Again',true)
assertEqual(data,writeData);

writeData = [1 2; 3 4; 5 6];
data = reader.readLeafData('AntRawData',true)
%assertEqual(data,writeData);

writeData = [1 2; 3 4; 5 7];
reader.writeLeafData('Again4',writeData);

reader.setRunName('againagain')
reader.writeLeafData('Again',writeData);
data = reader.readLeafData('Again',true)
assertEqual(data,writeData);

function noFile()
    filename = 'simpeele-test-data.h5';
    h5Reader(filename,'/');
    
    function [ret] = thereIsAFile()
    filename = 'simple-test-data.h5';
    h5Reader(filename,'/');
    ret = 12
    
function test_fileDoesNotExistThrown()
assertExceptionThrown(@noFile,'sofie:h5ReaderNotExist');
assertEqual(thereIsAFile(),12);

function test_listRuns()
filename = 'simple-test-data.h5';
runs = h5Reader.listRuns(filename);
assertEqual(runs,{'/adams', '/newpromove','/newpromovewithsync','/promovetest', '/sparktest', '/test','/vicon'});

function test_readStringColumn()
filename = 'simple-test-data.h5';
    reader = h5Reader(filename,'/test/goldenembeded/');
    theRunMeta = reader.readLeafData('RunMeta',false);
    converted = reader.convertColumnToStrings(theRunMeta.runNameLong);
    class(converted)
    display(converted)
    assertEqual(strcmp(converted,'/test/goldenembeded'),true);

function test_readDavisBDP_getRunManeuvers()
display('Testing the DavisFileReader.')
filename = 'test-bdp-data.h5';
theReader = bdpReader(filename);
theManeuvers = theReader.getRunManeuvers()
assertEqual(size(theManeuvers{1,1}),size(theManeuvers{1,2}))
theID = theManeuvers{1,1};
theManeuver = theManeuvers{1,2};
class(theID{1})
assertEqual(theID{1},0)
assertEqual(theManeuver{1},'Lane Change')


assertEqual(theID{181},180)
assertEqual(theManeuver{180},'System Test')
assertEqual(theManeuver{181},'Track Straight Line')

assertEqual(theID{707},706)
assertEqual(theManeuver{707},'Balance with Disturbance Blind')

theManeuvers = theReader.getRunManeuvers('Bump');
assertEqual(size(theManeuvers),[707 1])
assertEqual(any(theManeuvers),true)
assertEqual(theManeuvers(theManeuvers>0),[1;2;3])

theManeuvers = theReader.getRunManeuvers('NA');
assertEqual(size(theManeuvers),[707 1])
assertEqual(any(theManeuvers),true)
assertEqual(theManeuvers(theManeuvers>0),[4;544])

function test_readDavisBDP_readRunData()
    format long;
filename = 'test-bdp-data.h5';
theReader = bdpReader(filename);
[theTables,theTableNames] = theReader.readRunData(105,true);
assertEqual(size(theTables),[11252,20]);
theTableNames(1)
assertElementsAlmostEqual(theTables(1,1),4.980180797328980);
theTableNames(20)
assertElementsAlmostEqual(theTables(11252,20),   0.2259685154442572);

[theTables,theTableNames] = theReader.readRunData(8,false);
assertEqual(size(theTables),[12000,23]);
theTableNames(1)
assertElementsAlmostEqual(theTables(1,1),NaN);
theTableNames(20)
assertElementsAlmostEqual(theTables(12000,20),2.867771915030477);