import logging
import os
from os import unlink
import unittest

from numpy.core.numeric import ndarray
from sofiehdfformat.antpytableaccess.AntPyTableAccess import AntRawDataAccess
import sofiehdfformat.core.SofieAdamFile as adam
from sofiehdfformat.core.SofieCoreLogger import setLogger
from sofiehdfformat.core.SofieCsvAccess import SofieCsvAccess
import sofiehdfformat.core.SofieCsvFile as scf
from sofiehdfformat.core.SofieCsvPyTableAccess import SofieCsvPyTableAccess
from sofiehdfformat.core.SofiePyTableAccess import SofiePyTableAccess
class CoreTest (unittest.TestCase):
    def setUp(self):
        if os.path.isfile('simple-test-data.h5'):
            unlink('simple-test-data.h5')
        if os.path.isfile('output.csv'):
            unlink('output.csv')
        
    def tearDown(self):
        pass
    
    def test_createNiceRunName(self):
        runName = SofiePyTableAccess._createNiceRunName('/')   
        self.assertEqual('/',runName)
        runName = SofiePyTableAccess._createNiceRunName('//')
        self.assertEqual('/',runName)
        runName = SofiePyTableAccess._createNiceRunName('//')
        self.assertEqual('/',runName)

    def test_setRunName(self):
        writer = SofiePyTableAccess('simple-test-data.h5', '/test')
        writer.setRunName('goat')
        self.assertEqual(writer.getRuns('goat'), [])
        writer.setRunName('goat1', False)
        self.assertEqual(writer.getRuns('goat1'), [])
        self.assertEqual(writer.getRunName(), '/')

    def test_getRunName(self):
        writer = SofiePyTableAccess('simple-test-data.h5', '/test')
        self.assertEqual(writer.getRunName(), '/test')
        writer.setRunName('goat')
        self.assertEqual(writer.getRunName(), '/goat')
        self.assertEqual(writer.getRuns('goat'), [])
        writer.setRunName('goat1', False)
        self.assertEqual(writer.getRuns('goat1'), [])
        self.assertEqual(writer.getRunName(), '/')

    def test_deleteRun(self):
        writer = SofiePyTableAccess('simple-test-data.h5', '/test')
        self.assertEqual(writer.getRunName(), '/test')
        self.assertEqual(writer.getRuns('/'), ['/RunMeta'])
        self.assertFalse(writer.deleteRun('goat'))
        self.assertFalse(writer.deleteRun('test'))
        self.assertEqual(writer.getRuns('/'), ['/RunMeta'])
        self.assertFalse(writer.deleteRun('test', recursive=True))
        self.assertEqual(writer.getRuns('/'), ['/RunMeta'])
        writer.setRunName('/again', True)
        self.assertEqual(writer.getRuns('/'), ['/RunMeta'])
        self.assertFalse(writer.deleteRun('/again/', recursive=True))

    def test_deleteNode(self):
        writer = SofiePyTableAccess('simple-test-data.h5', '/test')
        self.assertEqual(writer.getRunName(), '/test')
        writer.setRunName('goat')
        self.assertEqual(writer.getRunName(), '/goat')
        self.assertEqual(writer.getRuns('goat'), [])
        writer.setRunName('goat1', False)
        self.assertEqual(writer.getRuns('goat1'), [])
        self.assertEqual(writer.getRunName(), '/')

    def test_meta(self):
        writer = SofiePyTableAccess('simple-test-data.h5', '/test')
        self.assertEqual(writer.getRuns('/'), ['/RunMeta'])
        data = writer.getRunMeta('test')
        logging.debug('Meta Data for the table:' + str(data))
        self.assertEqual(len(data),8)
        
        writer.setRunMeta('ourRun', 'The Description')
        data = writer.getRunMeta('ourRun')
        self.assertEqual(len(data), 8)
        logging.debug('The Data:'+str(data))
        self.assertEqual('ourRun', data['runName'])
        self.assertEqual('The Description', data['runNotes'])
        
        self.assertTrue(writer.setRunMeta('ourRun', 'The New Description'))
        data = writer.getRunMeta('ourRun')
        logging.debug(data)
        self.assertEqual(len(data), 8)
        logging.debug('The Data:'+str(data))
        self.assertEqual('ourRun', data['runName'])
        self.assertEqual('The New Description', data['runNotes'])

        data = writer.getRunMeta('test')
        self.assertEqual(len(data), 8)

        writer = SofiePyTableAccess('simple-test-data.h5', runName='/testagain',
        description='our description again')
        data = writer.getRunMeta('/testagain')
        self.assertEqual(len(data), 8)
        self.assertEqual('testagain', data['runName'])
        self.assertEqual('our description again', data['runNotes'])
        data = writer.getRunMeta('testagain')
        self.assertEqual(len(data), 8)
        self.assertEqual('testagain', data['runName'])
        self.assertEqual('our description again', data['runNotes'])

        self.assertIsNone(writer.setRunName('/'));
        self.assertIsNone(writer.setRunName('/'));
        self.assertIsNone(writer.setTable('/', ''));

    def test_sync(self):
        writer = SofiePyTableAccess('simple-test-data.h5', runName='/sync',
                description='our description')
        time1 = writer.writeSync(sample=11)
        time2 = writer.writeSync(sample=26)

        syncs = writer.getSyncs()
        logging.debug(syncs)
        self.assertFalse(syncs[0]['Timestamp'] == time2)
        self.assertEqual(syncs[0]['Sample'], 11)
        self.assertEqual(syncs[1]['Sample'], 26)
        self.assertEqual(syncs[0]['Timestamp'], time1)
        self.assertFalse(syncs[1]['Timestamp'] == time1)
        self.assertEqual(syncs[1]['Timestamp'], time2)
        
    def test_SofiePyTableAccess(self):
        logging.info('AntPyTableAccess Test')
        writer = SofiePyTableAccess('simple-test-data.h5', '/test')
        self.assertIsInstance(writer, SofiePyTableAccess, 'Root Does not exist')
        writer.close()
        writer = SofiePyTableAccess('simple-test-data.h5', '/test')
        self.assertIsInstance(writer, SofiePyTableAccess, 'Root Exists')

        writer = SofiePyTableAccess('simple-test-data.h5', '/testagain')
        self.assertIsInstance(writer, SofiePyTableAccess, 'New Root')

        writer = SofiePyTableAccess('simple-test-data.h5', '/testagain/new')
        self.assertIsInstance(writer, SofiePyTableAccess, 'New Leaf')

        writer = SofiePyTableAccess('simple-test-data.h5', '/testagain/new')
        self.assertIsInstance(writer, SofiePyTableAccess, 'Repeat Leaf')
        writer.setRunName('test/new2')
        writer.setRunName('testagain/new3/sub1')
        writer.setRunName('testagain/new3/sub1')
        writer.setRunName('testagain/new3/sub2')

        writer = AntRawDataAccess('simple-test-data.h5', '/test')
        writerData = ('1', '2', '3', '4', '5', '6', '7', '8', '9', 111111.101221)
        logging.info('Writing the following data: ' + str(writerData))
        writer.write(writerData)
        logging.info('Came Back from writing.')
        writer.close()

        writer = AntRawDataAccess('simple-test-data.h5', '/test/AntRawData')
        self.assertIsInstance(writer, AntRawDataAccess)
        
    def test_SofiePyTableAccess_NoneTypes(self):
        writer = SofieCsvPyTableAccess('simple-test-data.h5', '/test',fieldNames={'test1',
                                                                                 'test2'})
        writer.write({'test1':0.2,'test2':0.4})
        writer.write({'test1':None,'test2':0.4})
        writer.write({'test1':0.2,'test2':None})
        writer.write({'test1':0.2,'test2':None})

    def test_SofiePyTableAccessReadLine(self):
        logging.info('AntPyTableAccess ReadLine')
        writer = AntRawDataAccess('simple-test-data.h5', '/test')
        writerData = ('1', '2', '3', '4', '5', '6', '7', '8', '9', 111111.101221)
        logging.info('Writing the following data: ' + str(writerData))
        writer.write(writerData)
        logging.info('Came Back from writing.')
        writer.close()

        writerPacket = [int(number) for number in writerData if isinstance(number, str)]
        writer = AntRawDataAccess('simple-test-data.h5', '/test')

        #self.assertRaises(SetTheTableFirstException, writer.readline)
        data = writer.readlines();
        logging.debug('Returned data')
        logging.debug(data)
        logging.debug(type(data))
        data = list(data[0])
        logging.debug('The Data' + str(data))
        logging.debug(type(data))
        expected = writerPacket + [writerData[-1]]
        logging.debug('Expected Data' + str(expected))
        self.assertEqual(expected[0:8], data[0:8])
        self.assertEqual(float(expected[9]), float(data[9]))
        writerData1 = ('2', '2', '3', '4', '5', '6', '7', '8', '9', 222222.02221)
        writer.write(writerData1)
        writer.close()

        writer = AntRawDataAccess('simple-test-data.h5', '/test')
        data = writer.readlines();
        data = list(data[0])
        self.assertEqual(expected, data)

        dataR = writer.readlines();
        data = list(dataR[0])
        data1 = list(dataR[1])
        logging.debug(dataR)
        writerPacket1 = [int(number) for number in writerData1 if isinstance(number, str)]
        expected1 = writerPacket1 + [writerData1[-1]]
        self.assertEqual(expected, data)
        self.assertEqual(expected1, data1)

        logging.debug('Reading the rows:')
        for row in writer.table:
            logging.debug(row)
        logging.debug('Reading the rows Writer:')
        for row in writer:
            logging.debug(row['CumulativeSpeedRevolutionCountMSB'])
            
            self.assertEqual(2, row['BikeCadenceEventTimeLSB'])
            self.assertEqual(3, row['BikeCadenceEventTimeMSB'])
            self.assertEqual(4, row['CumulativeCadenceRevolutionCountLSB'])
            self.assertEqual(5, row['CumulativeCadenceRevolutionCountMSB'])
            self.assertEqual(6, row['BikeSpeedEventTimeLSB'])
            self.assertEqual(7, row['BikeSpeedEventTimeMSB'])
            self.assertEqual(8, row['CumulativeSpeedRevolutionCountLSB'])
            self.assertEqual(9, row['CumulativeSpeedRevolutionCountMSB'])
            logging.debug(row['Timestamp'])
            self.assertTrue((row['Timestamp'] == 111111.101221) or \
               (row['Timestamp'] == 222222.02221))
            self.assertTrue((row['ChanID'] == 1) or \
               (row['ChanID'] == 2))

        self.assertEqual(writer.getDuplicateWrites(), 0)
        writer.write(writerData1)
        self.assertEqual(writer.getDuplicateWrites(), 1)

        writer.allowDuplicates()
        writer.write(writerData1)
        self.assertEqual(writer.getDuplicateWrites(), 1)
        dataR = writer.readlines();
        self.assertEqual(len(dataR), 3)

        writer.doNotAllowDuplicates()
        writer.write(writerData1)
        self.assertEqual(writer.getDuplicateWrites(), 2)
        dataR = writer.readlines();
        self.assertEqual(len(dataR), 3)

    def test_sofiecsvfile_promove(self):
        csvFile = scf.open('test-data/test-promove.csv')
        csvWriter = SofieCsvAccess('output.csv')

        for row in csvFile:
            csvWriter.write(row)

        csvFile.close()
        csvWriter.close()

        csvFile = scf.open('test-data/test-promove.csv')
        csvFileNew = scf.open('output.csv')
        logging.debug(csvFileNew.reader)
        for row, newRow in zip(csvFile, csvFileNew):
            for key in row.keys():
                if row[key] == None:
                    self.assertEqual(newRow[key], '')
                else:
                    self.assertEqual(row[key], newRow[key])

    def test_sofiecsvfile_adams(self):
        logging.debug('Creating CSV FILE');
        csvFile = adam.open('test-data/test-adams.tab')
        logging.debug('CSVFILE CREATED.');
        csvWriter = SofieCsvAccess('output.csv')

        for row in csvFile:
            csvWriter.write(row)

        csvFile.close()
        csvWriter.close()

        csvFile = adam.open('test-data/test-adams.tab')
        csvFileNew = scf.open('output.csv')
        logging.debug(csvFileNew.reader)
        for row, newRow in zip(csvFile, csvFileNew):
            for key in row.keys():
                #logging.debug('"'+key+'"')
                self.assertTrue(key == 'Time      ' or  key == 'Roll angle'\
                    or key == 'Steering angle')
                if row[key] == None:
                    self.assertEqual(newRow[key], '')
                else:
                    self.assertEqual(row[key], newRow[key])
                    
    def test_makesafekey(self):
        csvWriter = SofiePyTableAccess('simple-test-data.h5')
        self.assertEqual('test19', csvWriter._makesafekey('test.1_9'))
        self.assertEqual('test19', csvWriter._makesafekey('_test.1_9_'))
        self.assertEqual('test19', csvWriter._makesafekey('.test.1_9.'))
        self.assertEqual('test19', csvWriter._makesafekey('__.test.1_9.__'))
        self.assertEqual('test19', csvWriter._makesafekey('__#$%^&*!#$%^^.test.1_9.__'))
    def test_makeSafeValue(self):
        csvWriter = SofiePyTableAccess('simple-test-data.h5')
        self.assertEqual('"test\.1\_9"', csvWriter._makeSafeValue('test.1_9'))
        self.assertEqual('"test\.1\_\\\n9"', csvWriter._makeSafeValue('test.1_\n9'))
        self.assertEqual(10, csvWriter._makeSafeValue(10))
        self.assertEqual(10, csvWriter._makeSafeValue('10'))
        self.assertEqual(10, csvWriter._makeSafeValue('10.000'))
        self.assertEqual(10, csvWriter._makeSafeValue('1e1'))

    def test_sofiecsvfile_adams_strangenames(self):
        logging.debug('Creating CSV FILE');
        csvFile = adam.open('test-data/test-adams-strangenames.tab')
        logging.debug('CSVFILE CREATED.');
        fieldNames = csvFile.getTableStructure(asList=True)
        logging.debug(fieldNames)
        csvWriter = SofieCsvPyTableAccess('simple-test-data.h5',
            fieldNames=fieldNames, runName='/hello/again/')

        for row in csvFile:
            logging.debug(row)
            csvWriter.write(row)

        csvFile.close()
        csvWriter.close()

        sofieReader = SofieCsvPyTableAccess('simple-test-data.h5',
            '/hello/again/CsvImportData')
        for row in sofieReader:
            logging.debug(row)
            self.assertTrue(row['Timer19'] >= 0)

    def test_setTable(self):
        writer = AntRawDataAccess('simple-test-data.h5', '/')

        self.assertIsNone(writer.setTable('/', 'HEELLO'));
        self.assertEqual(writer.getRun(), '/');

    def test_getRunsAndGetRunsMetaFromFile(self):
        filename ='simple-test-data.h5'
        writer = AntRawDataAccess(filename, '/test')
        #self.assertEqual(writer.getRuns(),[]);
        writerData = ('1', '2', '3', '4', '5', '6', '7', '8', '9', 111111.101221)
        logging.info('Writing the following data: ' + str(writerData))
        writer.write(writerData)
        logging.info('Came Back from writing.')
        writer.close()

        writer = AntRawDataAccess(filename, '/test')
        self.assertEqual(writer.getRuns(),
            ['/RunMeta','/test/AntRawData']);

        writer.setRunName('again');
        writerData = ('1', '2', '3', '4', '5', '6', '7', '8', '9', 111111.101221)
        logging.info('Writing the following data: ' + str(writerData))
        writer.write(writerData)
        logging.info('Came Back from writing.')

        self.assertEqual(writer.getRuns(),
            ['/RunMeta','/again/AntRawData', '/test/AntRawData']);
        self.assertEqual(writer.getRuns('again'),
            ['/again/AntRawData']);
        self.assertEqual(writer.getRuns('agai'),
            []);
        self.assertEqual(writer.getRuns('agai'),
            []);

        self.assertEqual(writer.getRuns('again/AntRawData'),
            ['/again/AntRawData']);
        writer.close()
        
        #Testing RUNMETA static functions.
        
        theRuns = SofieCsvPyTableAccess.getRunsInTheFile('simple-test-data.h5')
        logging.info('The RUNS: '+ '|'.join(theRuns))
        expected = {'runSubject':'Our Subject','runNotes':'Test\n\nttt'}
        SofieCsvPyTableAccess.setRunMetaInFile(filename,'again',expected)     

        theRunMeta =    SofieCsvPyTableAccess.getRunMetaInFile(filename,'again')     
        logging.info('The Run INFO: {0}'.format(theRunMeta))
        self.assertTrue(all(item in theRunMeta.items() for item in expected.items()))
        
    def test_twoWritersAtOnce(self):
        #ASYNC WRITING TO PYTABLES IS NOT RELIABLE.
        csvWriter = SofieCsvPyTableAccess('simple-test-data.h5',
            fieldNames=['test', 'test1'], runName='/hello/again/')
        csvWriter1 = SofieCsvPyTableAccess('simple-test-data.h5',
            fieldNames=['test', 'test1'], runName='/hello/again1/')
        
        theData = {'test':23, 'test1':24}
        theData1 = {'test':25, 'test1':26}
        
        csvWriter.write(theData)
        csvWriter1.write(theData1)
        csvWriter.close()
        csvWriter1.close()
        
        csvReader = SofieCsvPyTableAccess('simple-test-data.h5',
            fieldNames=['test', 'test1'], runName='/hello/again/')
        csvReader1 = SofieCsvPyTableAccess('simple-test-data.h5',
            fieldNames=['test', 'test1'], runName='/hello/again1/')
        for row,row1 in zip(csvReader,csvReader1):
            self.assertEqual(row['test'], 23)
            self.assertEqual(row['test1'], 24)
            self.assertEqual(row1['test'], 25)
            self.assertEqual(row1['test1'], 26)
        csvReader.close()
        csvReader1.close()

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(CoreTest))
    return suite

#setLogger(logging.DEBUG)

if __name__ == '__main__':
    setLogger(logging.DEBUG)
    runner = unittest.TextTestRunner()
    test_suite = suite()
    runner.run (test_suite)
